#!/usr/bin/env python
# --!-- coding: utf8 --!--
from PyQt5.QtCore import pyqtSignal, QTimer
from PyQt5.QtWidgets import QWidget, qApp

from manuskript.ui.editors.locker_ui import Ui_locker


class locker(QWidget, Ui_locker):
    locked = pyqtSignal()
    unlocked = pyqtSignal()
    lockChanged = pyqtSignal(bool)

    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self._btnText = None
        self._words = None
        self._target = None
        self._blackout = []

        self.timer = QTimer(self)
        self.timer.setSingleShot(True)
        self.timer.timeout.connect(self.unlock)
        self.timer.stop()
        self.timerSec = QTimer(self)
        self.timerSec.setInterval(500)
        self.timerSec.timeout.connect(self.updateBtnText)
        self.timerSec.stop()
        self.rbtnWordTarget.toggled.connect(self.spnWordTarget.setVisible)
        self.rbtnTimeTarget.toggled.connect(self.spnTimeTarget.setVisible)
        self.rbtnWordTarget.setChecked(True)
        self.spnTimeTarget.setVisible(False)

        self.btnLock.clicked.connect(self.lock)

    def lock(self):
        # Block others screens
        desktop = qApp.desktop()
        self._blackout.clear()
        if desktop.screenCount() > 1:
            for d in range(desktop.screenCount()):
                if desktop.screenNumber(self) != d:
                    w = QWidget()
                    w.setStyleSheet("background: black;")
                    w.move(desktop.screenGeometry(d).topLeft())
                    w.showFullScreen()
                    self._blackout.append(w)

        if self.rbtnWordTarget.isChecked():
            self._target = self._words + self.spnWordTarget.value()

        elif self.rbtnTimeTarget.isChecked():
            self.timer.setInterval(self.spnTimeTarget.value() * 1000 * 60)
            self.timer.start()
            self.timerSec.start()
            self.updateBtnText()

        self.setEnabled(False)
        self.locked.emit()
        self.lockChanged.emit(True)

    def unlock(self):
        # Remove black screens
        self._blackout.clear()

        self.setEnabled(True)
        self.btnLock.setText(self._btnText)
        self.timer.stop()
        self.timerSec.stop()
        self._target = None
        self.unlocked.emit()
        self.lockChanged.emit(False)

    def isLocked(self):
        return not self.isEnabled()

    def setWordCount(self, wc):
        self._words = wc
        if self.isLocked():
            self.updateBtnText()
            if self._target and self._words >= self._target:
                self.unlock()

    def updateBtnText(self):
        if not self._btnText:
            self._btnText = self.btnLock.text()

        # Time locked
        if self.timer.remainingTime() != -1:
            t = self.timer.remainingTime()
            t = int(t / 1000)
            if t > 60 * 60:
                text = self.tr("~{} h.").format(str(int(t / 60 / 60)))
            elif t > 60 * 5:
                text = self.tr("~{} mn.").format(str(int(t / 60)))
            elif t > 60:
                mn = int(t / 60)
                sec = t - 60 * mn
                text = self.tr("{}:{}").format(str(mn), str(sec))
            else:
                text = self.tr("{} s.").format(str(t))

            self.btnLock.setText(self.tr("{} remaining").format(
                    text))

        # Word locked
        elif self._target is not None:
            self.btnLock.setText(self.tr("{} words remaining").format(
                    self._target - self._words))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/views/sldImportance_ui.ui'
#
# Created: Thu Mar  3 18:52:22 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_sldImportance(object):
    def setupUi(self, sldImportance):
        sldImportance.setObjectName("sldImportance")
        sldImportance.resize(452, 354)
        self.horizontalLayout = QtWidgets.QHBoxLayout(sldImportance)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.sld = QtWidgets.QSlider(sldImportance)
        self.sld.setMinimum(0)
        self.sld.setMaximum(2)
        self.sld.setPageStep(1)
        self.sld.setProperty("value", 0)
        self.sld.setOrientation(QtCore.Qt.Horizontal)
        self.sld.setObjectName("sld")
        self.horizontalLayout.addWidget(self.sld)
        self.lbl = QtWidgets.QLabel(sldImportance)
        self.lbl.setObjectName("lbl")
        self.horizontalLayout.addWidget(self.lbl)

        self.retranslateUi(sldImportance)
        QtCore.QMetaObject.connectSlotsByName(sldImportance)

    def retranslateUi(self, sldImportance):
        _translate = QtCore.QCoreApplication.translate
        sldImportance.setWindowTitle(_translate("sldImportance", "Form"))
        self.lbl.setText(_translate("sldImportance", "TextLabel"))




#!/usr/bin/env python
# --!-- coding: utf8 --!--

from PyQt5.QtCore import QAbstractItemModel, QMimeData
from PyQt5.QtCore import QModelIndex
from PyQt5.QtCore import QSize
from PyQt5.QtCore import QVariant
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon, QFont
from PyQt5.QtWidgets import QTextEdit, qApp
from lxml import etree as ET
import re

from manuskript import enums


class abstractItem():

    # Enum kept on the class for easier access
    enum = enums.Abstract

    # Used for XML export
    name = "abstractItem"

    # Regexp from https://stackoverflow.com/questions/8733233/filtering-out-certain-bytes-in-python
    valid_xml_re = re.compile(u'[^\u0020-\uD7FF\u0009\u000A\u000D\uE000-\uFFFD\U00010000-\U0010FFFF]+')

    def __init__(self, model=None, title="", _type="abstract", xml=None, parent=None, ID=None):

        self._data = {}
        self.childItems = []
        self._parent = None
        self._model = model

        self.IDs = ["0"]  # used by root item to store unique IDs
        self._lastPath = ""  # used by loadSave version_1 to remember which files the items comes from,
                             # in case it is renamed / removed

        self._data[self.enum.title] = title
        self._data[self.enum.type] = _type

        if xml is not None:
            self.setFromXML(xml)

        if ID:
            self._data[self.enum.ID] = ID

        if parent:
            parent.appendChild(self)

    #######################################################################
    # Model
    #######################################################################

    def setModel(self, model):
        self._model = model
        for c in self.children():
            c.setModel(model)

    def index(self, column=0):
        if self._model:
            return self._model.indexFromItem(self, column)
        else:
            return QModelIndex()

    def emitDataChanged(self, cols=None, recursive=False):
        """
        Emits the dataChanged signal of the model, to signal views that data
        have changed.

        @param cols: an array of int (or None). The columns of the index that
                     have been changed.
        @param recursive: boolean. If true, all children will also emit the
                     dataChanged signal.
        """
        idx = self.index()
        if idx and self._model:
            if not cols:
                # Emit data changed for the whole item (all columns)
                self._model.dataChanged.emit(idx, self.index(len(self.enum)))

            else:
                # Emit only for the specified columns
                for c in cols:
                    self._model.dataChanged.emit(self.index(c), self.index(c))

            if recursive:
                for c in self.children():
                    c.emitDataChanged(cols, recursive=True)

    #######################################################################
    # Properties
    #######################################################################

    def title(self):
        return self._data.get(self.enum.title, "")

    def ID(self):
        return self._data.get(self.enum.ID)

    def columnCount(self):
        return len(self.enum)

    def type(self):
        return self._data[self.enum.type]

    #######################################################################
    # Parent / Children management
    #######################################################################

    def child(self, row):
        return self.childItems[row]

    def childCount(self):
        return len(self.childItems)

    def childCountRecursive(self):
        n = self.childCount()
        for c in self.children():
            n += c.childCountRecursive()
        return n

    def children(self):
        return self.childItems

    def row(self):
        if self.parent():
            return self.parent().childItems.index(self)
        return None

    def appendChild(self, child):
        self.insertChild(self.childCount(), child)

    def insertChild(self, row, child):
        self.childItems.insert(row, child)
        child._parent = self
        child.setModel(self._model)
        if not child.ID():
            child.getUniqueID()

    def removeChild(self, row):
        """
        Removes child at position `row` and returns it.
        @param row: index (int) of the child to remove.
        @return: the removed abstractItem
        """
        r = self.childItems.pop(row)
        # Disassociate the child from its parent and the model.
        r._parent = None
        r.setModel(None)
        return r

    def parent(self):
        return self._parent

    def path(self, sep=" > "):
        "Returns path to item as string."
        if self.parent().parent():
            return "{parent}{sep}{title}".format(
                parent=self.parent().path(),
                sep=sep,
                title=self.title())
        else:
            return self.title()

    def pathID(self):
        "Returns path to item as list of (ID, title)."
        if self.parent() and self.parent().parent():
            return self.parent().pathID() + [(self.ID(), self.title())]
        else:
            return [(self.ID(), self.title())]

    def level(self):
        """Returns the level of the current item. Root item returns -1."""
        if self.parent():
            return self.parent().level() + 1
        else:
            return -1

    def copy(self):
        """
        Returns a copy of item, with no parent, and no ID.
        """
        item = self.__class__(xml=self.toXML())
        item.setData(self.enum.ID, None)
        return item

    def siblings(self):
        if self.parent():
            return self.parent().children()
        return []

    ###############################################################################
    # IDS
    ###############################################################################

    def getUniqueID(self, recursive=False):
        self.setData(self.enum.ID, self._model.rootItem.findUniqueID())

        if recursive:
            for c in self.children():
                c.getUniqueID(recursive)

    def checkIDs(self):
        """This is called when a model is loaded.

        Makes a list of all sub-items IDs, that is used to generate unique IDs afterwards.
        """
        self.IDs = self.listAllIDs()

        if max([self.IDs.count(i) for i in self.IDs if i]) != 1:
            print("WARNING ! There are some items with same IDs:", [i for i in self.IDs if i and self.IDs.count(i) != 1])

        def checkChildren(item):
            for c in item.children():
                _id = c.ID()
                if not _id or _id == "0":
                    c.getUniqueID()
                checkChildren(c)

        checkChildren(self)

    def listAllIDs(self):
        IDs = [self.ID()]
        for c in self.children():
            IDs.extend(c.listAllIDs())
        return IDs

    def findUniqueID(self):
        IDs = [int(i) for i in self.IDs]
        k = 1
        while k in IDs:
            k += 1
        self.IDs.append(str(k))
        return str(k)

    #######################################################################
    # Data
    #######################################################################

    def data(self, column, role=Qt.DisplayRole):
        # Return value in self._data
        if role == Qt.DisplayRole or role == Qt.EditRole:
            return self._data.get(column, "")

        # Or return QVariant
        return QVariant()

    def setData(self, column, data, role=Qt.DisplayRole):
        # Setting data
        self._data[column] = data

        # Emit signal
        self.emitDataChanged(cols=[column]) # new in 0.5.0

    ###############################################################################
    # XML
    ###############################################################################

    # We don't want to write some datas (computed)
    XMLExclude = []
    # We want to force some data even if they're empty
    XMLForce = []

    def cleanTextForXML(self, text):
        return self.valid_xml_re.sub('', text)

    def toXML(self):
        """
        Returns a string containing the item (and children) in XML.
        By default, saves all attributes from self.enum and lastPath.
        You can define in XMLExclude and XMLForce what you want to be
        excluded or forcibly included.
        """
        item = ET.Element(self.name)

        for attrib in self.enum:
            if attrib in self.XMLExclude:
                continue
            val = self.data(attrib)
            if val or attrib in self.XMLForce:
                item.set(attrib.name, self.cleanTextForXML(str(val)))

        # Saving lastPath
        item.set("lastPath", self._lastPath)

        # Additional stuff for subclasses
        item = self.toXMLProcessItem(item)

        for i in self.childItems:
            item.append(ET.XML(i.toXML()))

        return ET.tostring(item)

    def toXMLProcessItem(self, item):
        """
        Subclass this to change the behavior of `toXML`.
        """
        return item

    def setFromXML(self, xml):
        root = ET.XML(xml)

        for k in self.enum:
            if k.name in root.attrib:
                self.setData(k, str(root.attrib[k.name]))

        if "lastPath" in root.attrib:
            self._lastPath = root.attrib["lastPath"]

        self.setFromXMLProcessMore(root)

        for child in root:
            if child.tag == self.name:
                item = self.__class__(self._model, xml=ET.tostring(child), parent=self)

    def setFromXMLProcessMore(self, root):
        """
        Additional stuff that subclasses must do with the XML to restore
        item.
        """
        return



#!/usr/bin/env python
# --!-- coding: utf8 --!--
import os
import shutil
import subprocess

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import qApp, QMessageBox
from PyQt5.QtGui import QCursor

from manuskript.converters import abstractConverter
from manuskript.functions import mainWindow

try:
    import markdown as MD
except ImportError:
    MD = None


class markdownConverter(abstractConverter):
    """
    Converter using python module markdown.
    """

    name = "python module markdown"

    @classmethod
    def isValid(self):
        return MD is not None

    @classmethod
    def convert(self, markdown):
        if not self.isValid:
            print("ERROR: markdownConverter is called but not valid.")
            return ""

        html = MD.markdown(markdown)
        return html



#!/usr/bin/env python
# --!-- coding: utf8 --!--
import os
import shutil
import subprocess

from PyQt5.QtCore import QSettings
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QCheckBox, QHBoxLayout, \
                            QLabel, QSpinBox, QComboBox, QLineEdit
from manuskript.ui.collapsibleGroupBox2 import collapsibleGroupBox2
from manuskript.ui import style


class abstractImporter:
    """
    abstractImporter is used to import documents into manuskript.

    The startImport function must be subclassed. It takes a filePath (str to
    the document to import), and must return `outlineItem`s.
    """

    name = ""
    description = ""
    fileFormat = ""  # File format accepted. For example: "OPML Files (*.opml)"
                     # For folder, use "<<folder>>"
    icon = ""
    engine = "Internal"

    def __init__(self):
        self.settingsList = []  # Keep the name of the settings in order
        self.settings = {}

    def startImport(self, filePath, parentItem, settingsWidget):
        """
        Takes a str path to the file/folder to import, and the settingsWidget
        returned by `self.settingsWidget()` containing the user set settings,
        and return `outlineItem`s.
        """
        pass

    @classmethod
    def isValid(cls):
        return False


    def settingsWidget(self, widget):
        """
        Takes a QWidget that can be modified and must be returned.
        """
        return widget

    def addPage(self, widget, title):
        """
        Convenience function to add a page to the settingsWidget `widget`, at
        the end.

        Returns the page widget.
        """
        w = QWidget(widget)
        w.setLayout(QVBoxLayout())
        widget.toolBox.insertItem(widget.toolBox.count(), w, title)
        widget.toolBox.layout().setSpacing(0)
        return w

    def addGroup(self, parent, title):
        """
        Adds a collapsible group to the given widget.
        """
        g = collapsibleGroupBox2(title=title)
        parent.layout().addWidget(g)
        g.setLayout(QVBoxLayout())
        return g

    def addSetting(self, name, type, label, widget=None,  default=None,
                   tooltip=None, min=None, max=None, vals=None, suffix=""):

        self.settingsList.append(name)
        self.settings[name] = self.setting(name, type, label, widget,  default,
                                         tooltip, min, max, vals, suffix)

    def widget(self, name):
        if name in self.settings:
            return self.settings[name].widget()

    def getSetting(self, name):
        if name in self.settings:
            return self.settings[name]

    def addSettingsTo(self, widget):
        """
        Adds all the settings to the given widget. Assume that the settings
        have not been called yet, so calling `.widget()` will create their
        widgets.
        """
        for name in self.settingsList:
            self.settings[name].widget(widget)


    class setting:
        """
        A class used to store setting, and display a widget for the user to
        modify it.
        """
        def __init__(self, name, type, label, widget=None,  default=None,
                     tooltip=None, min=None, max=None, vals=None, suffix=""):
            self.name = name
            self.type = type
            self.label = label
            self._widget = widget
            self.default = default
            self.min = min
            self.max = max
            self.vals = vals.split("|") if vals else []
            self.suffix = suffix
            self.tooltip = tooltip

        def widget(self, parent=None):
            """
            Returns the widget used, or creates it if not done yet. If parent
            is given, widget is inserted in parent's layout.
            """
            if self._widget:
                return self._widget

            else:

                if "checkbox" in self.type:
                    self._widget = QCheckBox(self.label)
                    if self.default:
                        self._widget.setChecked(True)
                    if parent:
                        parent.layout().addWidget(self._widget)

                elif "number" in self.type:
                    l = QHBoxLayout()
                    label = QLabel(self.label, parent)
                    label.setWordWrap(True)
                    l.addWidget(label, 8)
                    self._widget = QSpinBox()
                    self._widget.setValue(self.default if self.default else 0)
                    if self.min:
                        self._widget.setMinimum(self.min)
                    if self.max:
                        self._widget.setMaximum(self.max)
                    if self.suffix:
                        self._widget.setSuffix(self.suffix)
                    l.addWidget(self._widget, 2)
                    if parent:
                        parent.layout().addLayout(l)

                elif "combo" in self.type:
                    l = QHBoxLayout()
                    label = QLabel(self.label, parent)
                    label.setWordWrap(True)
                    l.addWidget(label, 6)
                    self._widget = QComboBox()
                    self._widget.addItems(self.vals)
                    if self.default:
                        self._widget.setCurrentText(self.default)
                    l.addWidget(self._widget, 2)
                    if parent:
                        parent.layout().addLayout(l)

                elif "text" in self.type:
                    l = QHBoxLayout()
                    label = QLabel(self.label, parent)
                    label.setWordWrap(True)
                    l.addWidget(label, 5)
                    self._widget = QLineEdit()
                    self._widget.setStyleSheet(style.lineEditSS())
                    if self.default:
                        self._widget.setText(self.default)
                    l.addWidget(self._widget, 3)
                    if parent:
                        parent.layout().addLayout(l)

                elif "label" in self.type:
                    self._widget = QLabel(self.label, parent)
                    self._widget.setWordWrap(True)
                    if parent:
                        parent.layout().addWidget(self._widget)

                if self.tooltip:
                    self._widget.setToolTip(self.tooltip)

                return self._widget

        def value(self):
            """
            Return the value contained in the widget.
            """
            if not self._widget:
                return self.default

            else:

                if "checkbox" in self.type:
                    return self._widget.isChecked()

                elif "number" in self.type:
                    return self._widget.value()

                elif "combo" in self.type:
                    return self._widget.currentText()

                elif "text" in self.type:
                    return self._widget.text()






#!/usr/bin/env python
# --!-- coding: utf8 --!--
from PyQt5.QtWidgets import QListView

from manuskript import settings
from manuskript.functions import findBackground
from manuskript.ui.views.corkDelegate import corkDelegate
from manuskript.ui.views.dndView import dndView
from manuskript.ui.views.outlineBasics import outlineBasics


class corkView(QListView, dndView, outlineBasics):
    def __init__(self, parent=None):
        QListView.__init__(self, parent)
        dndView.__init__(self, parent)
        outlineBasics.__init__(self, parent)

        self.setResizeMode(QListView.Adjust)
        self.setWrapping(True)
        self.setItemDelegate(corkDelegate())
        self.setSpacing(5)
        self.setVerticalScrollMode(self.ScrollPerPixel)
        self.setFlow(self.LeftToRight)
        self.setSelectionBehavior(self.SelectRows)
        self.updateBackground()

    def updateBackground(self):
        if settings.corkBackground["image"] != "":
            img = findBackground(settings.corkBackground["image"])
        else:
            # No background image
            img = ""
        self.setStyleSheet("""QListView {{
            background:{color};
            background-image: url({url});
            background-attachment: fixed;
            }}""".format(
                color=settings.corkBackground["color"],
                url=img.replace("\\", "/")
        ))

    def dragMoveEvent(self, event):
        dndView.dragMoveEvent(self, event)
        QListView.dragMoveEvent(self, event)

    def mouseReleaseEvent(self, event):
        QListView.mouseReleaseEvent(self, event)
        outlineBasics.mouseReleaseEvent(self, event)
        
    def mouseDoubleClickEvent(self, event):
        if self.selectedIndexes() == []:
            idx = self.rootIndex()
            parent = idx.parent()
            
            from manuskript.functions import MW
            MW.openIndex(parent)
            #self.setRootIndex(parent)
        else:
            r = QListView.mouseDoubleClickEvent(self, event)



#!/usr/bin/env python
# --!-- coding: utf8 --!--

import collections

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QStyledItemDelegate, QLineEdit, QMenu, QAction


class plotDelegate(QStyledItemDelegate):
    def __init__(self, parent=None):
        QStyledItemDelegate.__init__(self, parent)

    def sizeHint(self, option, index):
        s = QStyledItemDelegate.sizeHint(self, option, index)
        if s.width() < 200:
            s.setWidth(200)
        return s

    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        editor.setFrame(False)
        editor.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
        return editor

    def setEditorData(self, editor, index):
        editor.setText(index.model().data(index))

        self.txt = editor
        self.menu = QMenu(editor)

        plotsTypes = collections.OrderedDict({
            self.tr("General"): [
                self.tr("Promise"),
                self.tr("Problem"),
                self.tr("Progress"),
                self.tr("Resolution")
            ],
            self.tr("Try / Fail"): [
                self.tr("No and"),
                self.tr("Yes but"),
            ],
            self.tr("Freytag's pyramid"): [
                self.tr("Exposition"),
                self.tr("Rising action"),
                self.tr("Climax"),
                self.tr("Falling action"),
                self.tr("Resolution"),
            ],
            self.tr("Three acts"): [
                self.tr("1. Setup"),
                self.tr("1. Inciting event"),
                self.tr("1. Turning point"),
                "---",
                self.tr("2. Choice"),
                self.tr("2. Reversal"),
                self.tr("2. Disaster"),
                "---",
                self.tr("3. Stand up"),
                self.tr("3. Climax"),
                self.tr("3. Ending"),
            ],

            self.tr("Hero's journey"): [
                self.tr("Ordinary world"),
                self.tr("Call to adventure"),
                self.tr("Refusal of the call"),
                self.tr("Meeting with mentor"),
                self.tr("Crossing the Threshold"),
                self.tr("Tests"),
                self.tr("Approach"),
                self.tr("Abyss"),
                self.tr("Reward / Revelation"),
                self.tr("Transformation"),
                self.tr("Atonement"),
                self.tr("Return"),
            ],
        })

        for name in plotsTypes:
            m = QMenu(name, self.menu)

            for sub in plotsTypes[name]:
                if sub == "---":
                    m.addSeparator()
                else:
                    a = QAction(sub, m)
                    a.triggered.connect(self.submit)
                    m.addAction(a)

            self.menu.addMenu(m)

        editor.addAction(QIcon.fromTheme("list-add"), QLineEdit.LeadingPosition).triggered.connect(self.popupMenu)

    def setModelData(self, editor, model, index):
        val = editor.text()
        model.setData(index, val)

    def popupMenu(self):
        act = self.sender()
        self.menu.popup(self.txt.parent().mapToGlobal(self.txt.geometry().bottomLeft()))

    def submit(self):
        act = self.sender()
        self.txt.setText(act.text())
